'use client'

import React, { useEffect, useRef, useState } from "react"
import * as R from 'ramda'
import markdownit from 'markdown-it'
import { v4 as uuid4 } from 'uuid'
import Chater from './Chater'
import Sender from '@/app/components/ChatSender'
import { useReducerAtom } from "jotai/utils"
import { getChatResponse, template } from "@/lib/utils"
import { useSession } from "next-auth/react"
import { useAtomValue } from "jotai"
import { cardInstantAtomFamily, curStepAtom, instantDataAtom, stepsNodesAtom } from "../../store"


const Agent = ({ node, asideInstantAtom }) => {
  const { data: session } = useSession()
  const [asideInstant, dispatchAsideInstant] = useReducerAtom(asideInstantAtom, (prev, payload) => ({ ...prev, ...payload }))
  const [messages, setMessages] = useState(asideInstant?.messages ?? [])
  // const messageItem = useRef<unknown>(null)
  const ctrlRef = useRef<AbortController>(null)
  const [sessionName, setSessionName] = useState(asideInstant?.sessionName ?? uuid4())

  const stepsNodes = useAtomValue(stepsNodesAtom)
  const curStep = useAtomValue(curStepAtom)
  const instantData = useAtomValue(instantDataAtom)

  const [input, setInput] = useState('')
  const [isSending, setIsSending] = useState(false)

  useEffect(() => {
    dispatchAsideInstant({ messages, sessionName })
  }, [messages, sessionName])

  const onSend = async ({ text, ignoreQuestionMessage = false }: { text: string, ignoreQuestionMessage?: boolean }) => {
    const assistantId = R.path(['properties', 'item', 'assistant_id'], node)
    const newMessages = []
    if (!ignoreQuestionMessage) {
      newMessages.push({ type: 'md', role: 'user', content: text })
    }
    const message = { type: 'md', role: 'assistant', content: '', isLoading: true }
    newMessages.push(message)
    setMessages((prev) => [...prev, ...newMessages])
    // messageItem.current = message
    try {
      const [chunks, ctrl] = getChatResponse({
        text,
        assistantId,
        sessionName,
        userId: session?.user?.id,
        // FIXME
        // userId: '1c9dc4b-d95f-11ea-af4c-52540005ab01'
      })
      ctrlRef.current = ctrl
      for await (const chunk of chunks) {
        message.content += chunk;
        setMessages((prev) => [...prev])
      }
    } catch (e) {
      message.isError = true
      message.error = e.message
    } finally {
      setMessages((prev) => [...prev])
      message.isLoading = false
      setIsSending(false)
      return message
    }
  }

  const onFirstSend = async () => {
    const prevNodes = R.slice(0, curStep, stepsNodes)
    const prevFormNodes = R.filter(R.propEq('form_card', 'type'), prevNodes)
    const prevFormInstants = R.map(_node => instantData?.[_node.id], prevFormNodes)
    const formContext = R.join('\n\n', R.map(
      formInstant => { return R.join('\n', R.map(fieldInstant => `${fieldInstant.label}: ${fieldInstant.value}`, R.values(formInstant.content ?? {}))) },
      prevFormInstants
    ))

    let nodeContext = R.pathOr('', ['properties', 'dispo_desc'], node)
    const replacementArray = R.pathOr('', ['properties', 'dispo_arr'], node)
    nodeContext = R.reduce(
      (prev, { desc, id }: { desc: string, id: string }) => {
        const to = instantData?.[id]?.content ?? ''
        return R.replace(`[${desc}]`, to, prev)
      },
      nodeContext,
      replacementArray
    )
    const text = `
    ${formContext}
    ${nodeContext}
    `
    setIsSending(true)
    const message = await onSend({ text, ignoreQuestionMessage: true })
    dispatchCardInstantData({ content: markdownit().render(message.content) })
  }

  useEffect(() => {
    if (!messages?.length) {
      onFirstSend()
    }
  }, [])

  const onCommit = async () => {
    const text = input
    setInput('')
    setIsSending(true)
    await onSend({ text })
  }

  const cardInstantAtom = cardInstantAtomFamily(node)

  const dataReducer = (prev, payload) => {
    return { ...prev, ...payload }
  }

  const [, dispatchCardInstantData] = useReducerAtom(cardInstantAtom, dataReducer)

  const onAccept = (message) => {
    const content = markdownit().render(message.content)
    dispatchCardInstantData({ content })
  }

  const onStop = async () => {
    ctrlRef.current?.abort()
  }

  const effectButtons = []
  if (!isSending) {
    effectButtons.push(
      <button key="rerun" className='btn btn-xs' onClick={() => {
        setMessages([])
        onFirstSend()
      }}>重新运行</button>,
    )
  }

  return (
    <div className="w-full h-full flex relative">
      <Chater messages={messages} node={node} onAccept={onAccept}></Chater>

      <Sender input={input} onInput={setInput} isSending={isSending} onStop={onStop} onCommit={onCommit} effectButtons={effectButtons} />
    </div>
  )
}

export default React.memo(React.forwardRef(Agent))