'use client'

import React, { useMemo } from 'react'
import Markdown from 'react-markdown'
import { twMerge } from 'tailwind-merge'
import * as R from 'ramda'
import { BsXCircle } from "react-icons/bs";



type IMessage = {
  type: 'md' | 'custom';
  role: 'user' | 'assistant';
  content: string;
  isLoading?: boolean;
  isError?: boolean;
  error?: string;
}
type IMessages = IMessage[]

const Chater = ({ messages, node, onAccept }: { messages: IMessages }) => {
  const reversedMessages = useMemo(() => {
    return R.reverse(messages)
  }, [messages])


  return (
    <div className="flex flex-col-reverse w-full overflow-auto pb-[90px]">
      {
        reversedMessages.map((message, i) => (
          <div key={i} className={twMerge("chat", message.role === 'assistant' ? "chat-start" : 'chat-end')}>
            <div className="chat-image avatar placeholder">
              {message.role === 'assistant'
                ? (
                  <div className="w-10 rounded-full">
                    <img
                      alt="avatar"
                      src="/images/assistant-avatar.png" />
                  </div>
                )
                : (
                  <div className="w-10 rounded-full bg-neutral text-neutral-content">
                    <span className="text-xl">我</span>
                  </div>
                )}
            </div>
            <div className={twMerge("chat-bubble prose prose-sm !prose-invert", message.role === 'assistant' ? "chat-bubble-primary" : "")}>
              <Markdown>
                {message?.content}
              </Markdown>
              {
                message.role === 'assistant'
                  ? message?.isLoading
                    ? <span className="loading loading-dots loading-sm"></span>
                    : message.isError
                      ? (
                        <>
                          <div className="divider my-0"></div>
                          <div role="alert" className="alert alert-error py-1">
                            <BsXCircle />
                            <span>
                              {message.error}
                            </span>
                          </div>
                        </>
                      )
                      : node.type === 'UserTask'
                        ? (
                          <>
                            <div className="divider my-0"></div>
                            <button className="btn btn-xs btn-neutral" onClick={() => onAccept?.(message)}>采纳</button>
                          </>
                        )
                        : null
                  : null
              }
            </div>
          </div>
        ))
      }
    </div>
  )
}

export default React.memo(Chater)